<?php
/**
 * @file
 *   Admin UI include.
 */

/**
 * UI Controller
 */
class EntityTemplateUIController extends EntityDefaultUIController {

  /**
   * {@inheritdoc}
   */
  public function hook_menu() {
    $items = array();
    $id_count = count(explode('/', $this->path));
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%'.$this->entityType;

    $items[$this->path . '/add'] = array(
      'title' => 'Create a Template',
      'description' => 'Create a new Entity Template',
      'page callback' => 'entity_template_create_page_wrapper',
      'access callback' => 'entity_template_access',
      'access arguments' => array('create'),
      'type' => MENU_LOCAL_ACTION,
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
    );

    $items[$this->path . '/add/%'] = array(
      'title' => 'Create a Template',
      'description' => 'Create a new Entity Template',
      'page callback' => 'entity_template_create_page_wrapper',
      'page arguments' => array($id_count + 1),
      'access callback' => 'entity_template_access',
      'access arguments' => array('create'),
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
      'type' => MENU_CALLBACK,
    );

    $items[$this->path . '/manage/%entity_template'] = array(
      'title callback' => 'entity_label',
      'title arguments' => array('entity_template', $id_count + 1),
      'description' => 'Edit a template',
      'page callback' => 'drupal_get_form',
      'page arguments' => array('entity_template_form', $id_count + 1),
      'access callback' => 'entity_template_access',
      'access arguments' => array('edit', $id_count + 1),
      'type' => MENU_CALLBACK,
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
    );

    $items[$this->path . '/manage/%entity_template/edit'] = array(
      'title' => 'Edit',
      'description' => 'Edit a template',
      'type' => MENU_DEFAULT_LOCAL_TASK,
    );

    // Clone form, a special case for the edit form.
    $items[$this->path . '/manage/%entity_template/clone'] = array(
      'title' => 'Clone',
      'page callback' => 'entity_ui_get_form',
      'page arguments' => array($this->entityType, $id_count + 1, 'clone'),
      'access callback' => 'entity_access',
      'access arguments' => array('create', $this->entityType),
      'type' => MENU_LOCAL_TASK,
    );

    // Revision form.
    $items[$this->path . '/manage/%entity_template/revision/%'] = array(
      'title' => 'Revision',
      'page callback' => 'drupal_get_form',
      'page arguments' => array('entity_template_revision_form', $id_count + 1),
      'access callback' => 'entity_template_access',
      'access arguments' => array('edit', $id_count + 1),
      'load arguments' => array($id_count + 3),
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
    );

    // Menu item for operations like revert and delete.
    $items[$this->path . '/manage/%entity_template/%'] = array(
      'page callback' => 'drupal_get_form',
      'page arguments' => array($this->entityType . '_operation_form', $this->entityType, $id_count + 1, $id_count + 2),
      'access callback' => 'entity_access',
      'access arguments' => array('delete', $this->entityType, $id_count + 1),
      'file' => 'includes/entity.ui.inc',
    );

    if (!empty($this->entityInfo['exportable'])) {
      // Menu item for importing an entity.
      $items[$this->path . '/import'] = array(
        'title' => t('Import a Template'),
        'page callback' => 'drupal_get_form',
        'page arguments' => array($this->entityType . '_operation_form', $this->entityType, NULL, 'import'),
        'access callback' => 'entity_access',
        'access arguments' => array('create', $this->entityType),
        'file' => 'includes/entity.ui.inc',
        'type' => MENU_LOCAL_ACTION,
      );
    }

    $items['entity_template/selector/%/%/%/%/%'] = array(
      'page callback' => 'entity_template_selector_autocomplete',
      'page arguments' => array(2,3,4,5,6),
      'access callback' => 'entity_template_access',
      'access arguments' => array('create'),
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
    );

    $items['admin/config/content/entity_template'] = array(
      'title' => t('Entity Templates'),
      'descripton' => t('Configure the Entity Template System'),
      'page callback' => 'drupal_get_form',
      'page arguments' => array('entity_template_config_form'),
      'access arguments' => array('administer entity_templates'),
      'file' => 'entity_template.admin.inc',
      'file path' => drupal_get_path('module', 'entity_template'),
      'type' => MENU_NORMAL_ITEM,
    );

    $items['admin/config/content/entity_template/settings'] = array(
      'title' => t('Settings'),
      'descripton' => t('Configure the Entity Template System'),
      'type' => MENU_DEFAULT_LOCAL_TASK,
    );

    return $items;
  }

}

/**
 * Entity Template Config Page Callback.
 */
function entity_template_config_form($form, &$form_state) {
  $access_groups = entity_template_access_groups();

  $default = '';
  foreach ($access_groups as $group => $label) {
    $default .= "{$group}|{$label}\n";
  }
  $form['access_groups'] = array(
    '#title' => t('Access Groups'),
    '#description' => t('Add and remove access groups. Access groups can be assigned permissions.'),
    '#type' => 'textarea',
    '#default_value' => $default,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Configuration'),
  );

  return $form;
}

/**
 * Entity Template Config Form.
 */
function entity_template_config_form_submit($form, &$form_state) {
  $lines = explode("\n", $form_state['values']['access_groups']);
  $groups = array();
  foreach ($lines as $line) {
    [$group, $label] = explode('|', $line);
    $groups[$group] = $label;
  }

  variable_set('entity_template_access_groups', $groups);
}

/**
 * Create Page Wrapper.
 */
function entity_template_create_page_wrapper($entity_type = FALSE, $bundle = FALSE) {
  $values = array();
  if ($entity_type) {
    $values['entity_type'] = $entity_type;
  }
  if ($bundle) {
    $values['bundle'] = $bundle;
  }
  $values['api'] = 2;
  $template = entity_create('entity_template', $values);
  return drupal_get_form('entity_template_form', $template, 'create');
}

/**
 * Entity template revision form.
 */
function entity_template_revision_form($form, &$form_state, $revision) {
  $form_state['revision'] = $revision;
  $form = entity_template_form($form, $form_state, $revision);
  unset($form['actions']);
  $form['revisions']['#weight'] = -10;
  $form['revisions']['#title'] = t('Revision Information');
  $form['revisions']['#collapsible'] = FALSE;

  $form['revisions']['summary'] = array(
    '#weight' => -5,
    '#type' => 'item',
    '#markup' => t(
      'Created by @user on @date.<br />Message: !message',
      array(
        '@user' => format_username(user_load($revision->uid)),
        '@date' => date('Y/m/d H:i:s', $revision->timestamp),
        '!message' => $revision->log,
      )
    ),
  );

  $form['revisions']['revert'] = array(
    '#type' => 'submit',
    '#value' => t('Revert to this Revision'),
    '#submit' => array(
      'entity_template_revision_form_revert_submit',
    ),
  );

  return $form;
}

/**
 * Entity template revert to revision option.
 */
function entity_template_revision_form_revert_submit($form, &$form_state) {
  $revision = $form_state['revision'];
  $new_revision = clone $revision;
  $new_revision->default_revision = TRUE;
  $new_revision->log = t('Reverted to @vid:', array('@vid' => $new_revision->vid)) . ' ' . $form_state['values']['log'];
  $new_revision->is_new_revision = TRUE;
  $new_revision->timestamp = REQUEST_TIME;

  $new_revision->save();

  drupal_set_message(t('Template Successfully Reverted.'));
  $form_state['redirect'] = 'admin/content/templates/manage/'.$new_revision->name;
}

/**
 * Form callback for ck_tasks.
 */
function entity_template_form($form, &$form_state, $template, $op = 'edit') {
  form_load_include($form_state, 'inc', 'entity_template', 'entity_template.admin');
  $form['#template'] = $form_state['template'] = $template;

  if (!empty($template->label)) {
    drupal_set_title(t('@type Template: @label', [
      '@type' => entity_get_info($template->entity_type)['label'] ?? 'Entity',
      '@label' => $template->label,
    ]), PASS_THROUGH);
  }

  // Update values if they have been set.
  if (!empty($form_state['input']['entity_type'])) {
    // If the template entity type changes clear the template form.
    if ($template->entity_type != $form_state['input']['entity_type']) {
      $template->bundle = NULL;
      $template->template = NULL;
      unset($form_state['template_form']);
    }
    $template->entity_type = $form_state['input']['entity_type'];
  }
  if (!empty($form_state['input']['bundle'])) {
    // If the template bundle changes clear the template form.
    if ($template->bundle != $form_state['input']['bundle']) {
      $template->template = NULL;
      unset($form_state['template_form']);
    }
    $template->bundle = $form_state['input']['bundle'];
  }

  if ($op == 'clone') {
    $template->label .= ' (Cloned)';
  }

  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Basic Information'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($template->is_new),
  );

  $form['basic']['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => $template->label,
    '#description' => t('The human-readable name of this template.'),
    '#required' => TRUE,
    '#size' => 30,
  );
  // Machine-readable template name.
  $form['basic']['name'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($template->name) ? $template->name : '',
    '#maxlength' => 32,
    '#disabled' => !($op == 'create' || $op == 'clone'),
    '#machine_name' => array(
      'exists' => 'entity_template_load',
      'source' => array('basic', 'label'),
    ),
    '#description' => t('A unique machine-readable name for this template. It must only contain lowercase letters, numbers, and underscores.'),
  );

  // Entity Type options.
  $form['basic']['entity_type'] = array(
    '#type' => 'select',
    '#options' => entity_template_entity_type_options_list(),
    '#title' => t('Entity Type'),
    '#default_value' => $template->entity_type,
    '#required' => TRUE,
    '#disabled' => ($op != 'create' && $op != 'clone'),
  );
  // We don't add the #ajax if the element is disabled partly because it won't
  // have any effect anyway and mainly because it stops the parameter table
  // from working.
  // @todo: Work out why!
  if (!$form['basic']['entity_type']['#disabled']) {
    $form['basic']['entity_type']['#ajax'] = array(
      'callback' => 'entity_template_form_update_entity_type',
      'wrapper' => 'bundle-select',
    );
  }

  // Bundle Options
  $bundle_options = array();
  if (!empty($template->entity_type)) {
    $bundle_options = entity_template_bundle_options_list($template->entity_type);
  }
  $form['basic']['bundle'] = array(
    '#type' => 'select',
    '#options' => $bundle_options,
    '#title' => t('Bundle'),
    '#default_value' => $template->bundle,
    '#required' => TRUE,
    '#disabled' => ($op != 'create' && $op != 'clone'),
    '#prefix' => '<div id="bundle-select">',
    '#suffix' => '</div>',
  );
  if (!$form['basic']['bundle']['#disabled']) {
    $form['basic']['bundle']['#ajax'] = array(
      'callback' => 'entity_template_form_update_bundle',
      'wrapper' => 'template-form',
    );
  }

  // Tags
  $form['basic']['tags'] = array(
    '#type' => 'textfield',
    '#title' => t('Tags'),
    '#default_value' => drupal_implode_tags($template->tags),
    '#autocomplete_path' => 'template/autocomplete/tags',
    '#autocomplete_multiple' => TRUE,
  );

  // Attach Fields
  field_attach_form('entity_template', $template, $form['basic'], $form_state);

  // Build parts to validate.
  $limit_validation_errors = array(
    array('bundle'),
    array('entity_type'),
    array('label'),
    array('name'),
    array('template'),
    array('settings'),
    array('log'),
    array('tags'),
  );

  // Add field api fields to validation list.
  $field_names = array_keys(field_info_instances('entity_template', 'entity_template'));
  foreach ($field_names as $field_name) {
    $limit_validation_errors[] = array($field_name);
  }

  $form['parameters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Parameters'),
    '#description' => t('Declare additional variables that must be passed in to use this template.'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($form_state['open_params_fieldset']),
  );

  $form['parameters']['table'] = array(
    '#theme' => 'table',
    '#parents' => array('parameters'),
    '#tree' => TRUE,
    '#header' => array(
      t('Label'),
      t('Machine Name'),
      array('data' => t('Type'), 'colspan' => 2),
      t('Optional'),
      t('Operations'),
    ),
    '#rows' => array(),
  );
  $data_items = rules_fetch_data('data_info');
  foreach ($template->parameters as $key => $info) {
    $row = array();
    $row['label'] = array(
      '#markup' => "{$info['label']} <small>({$key})",
    );

    $row['type'] = array(
      '#markup' => ucfirst($data_items[$info['type']]['label']),
    );
    if (!empty($info['bundle'])) {
      $einfo = entity_get_info($info['type']);
      $row['type']['#markup'] .= ' ('.$einfo['bundles'][$info['bundle']]['label'].')';
    }
    if (!empty($info['granularity'])) {
      $row['type']['#markup'] .= ' ('.($info['granularity'] == 'datetime' ? 'Date & Time' : 'Date Only').')';
    }

    $row['optional'] = array(
      '#markup' => !empty($info['optional']) ? 'Yes' : 'No',
    );

    $row['operations']['remove'] = array(
      '#type' => 'submit',
      '#value' => t('Remove @param', array('@param' => $info['label'])),
      '#param_key' => $key,
      '#limit_validation_errors' => array(),
      '#submit' => array('entity_template_form_submit_remove_parameter'),
    );

    $form['parameters']['table'][$key] = array(
      'label' => &$row['label'],
      'type' => &$row['type'],
      'optional' => &$row['optional'],
      'operations' => &$row['operations'],
      '#value' => $info,
    );

    $form['parameters']['table']['#rows'][$key] = array(
      'label' => array('colspan' => 2, 'data' => &$row['label']),
      'type' => array('colspan' => 2, 'data' => &$row['type']),
      'optional' => array('data' => &$row['optional']),
      'operations' => array('data' => &$row['operations']),
    );

    unset($row);
    $limit_validation_errors[] = array('parameters',$key);
  }

  // If this is an entity type then give the options to specify a bundle.
  if (!empty($form_state['input']['parameters']['add_new']['type'])) {
    $type = $form_state['input']['parameters']['add_new']['type'];
    $entity_type = FALSE;
    if ($data_items[$type]['parent'] == 'entity') {
      $entity_type = $type;
    }
    else if (($list_item_type = entity_property_list_extract_type($type))
             && $data_items[$list_item_type]['parent'] == 'entity') {
      $entity_type = $list_item_type;
    }

    $einfo = entity_get_info($entity_type);
    if (count($einfo['bundles']) > 1) {
      foreach ($einfo['bundles'] as $param_bundle => $binfo) {
        $param_bundle_options[$param_bundle] = $binfo['label'];
      }
    }
  }

  $row = array();
  $row['label'] = array(
    '#type' => 'textfield',
    '#title' => t('Label'),
    '#title_display' => 'attribute',
  );
  $row['machine_name'] = array(
    '#type' => 'machine_name',
    '#title' => t('Machine Name'),
    '#title_display' => 'invisible',
    '#machine_name' => array(
      'source' => array('parameters', 'add_new', 'label'),
      'exists' => 'entity_template_check_param_machine_name',
    ),
    '#requried' => FALSE,
  );
  $row['type'] = array(
    '#type' => 'select',
    '#options' => array(0 => '--') + RulesPluginUI::getOptions('data'),
    '#title' => t('Type'),
    '#title_display' => 'attribute',
    '#ajax' => array(
      'wrapper' => 'new-param-type-bundle',
      'callback' => 'entity_template_form_ajax_new_param_type',
    ),
    '#limit_validation_errors' => array(
      array('parameters', 'add_new'),
    ),
  );

  $row['info'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'new-param-type-bundle',
    ),
  );
  if (!empty($param_bundle_options)) {
    $row['info']['bundle'] = array(
      '#type' => 'select',
      '#options' => $param_bundle_options,
      '#title' => t('Bundle'),
      '#title_display' => 'attribute',
      '#description' => t('Select the bundle if known.'),
      '#empty_option' => t('- Any Bundle -'),
      '#parents' => array('parameters', 'add_new', 'bundle'),
    );
  }
  else if (!empty($form_state['input']['parameters']['add_new']['type']) && $form_state['input']['parameters']['add_new']['type'] == 'date') {
    $row['info']['granularity'] = array(
      '#type' => 'select',
      '#title' => t('Granularity'),
      '#title_display' =>'attribute',
      '#options' => array(
        'date' => t('Date Only'),
        'datetime' => t('Date & Time'),
      ),
      '#parents' => array('parameters', 'add_new', 'bundle'),
    );
  }

  $row['optional'] = array(
    '#type' => 'checkbox',
    '#title' => t('Is this parameter optional?'),
    '#title_display' => 'attribute',
    '#default_value' => FALSE,
  );
  $row['operations']['#type'] = 'container';
  $row['operations']['add_param'] = array(
    '#type' => 'submit',
    '#value' => t('Add Parameter'),
    '#validate' => array('entity_template_form_validate_add_parameter'),
    '#submit' => array('entity_template_form_submit_add_parameter'),
    '#limit_validation_errors' => array(
      array('parameters', 'add_new'),
    ),
  );
  $form['parameters']['table']['add_new'] = array(
    'label' => &$row['label'],
    'machine_name' => &$row['machine_name'],
    'type' => &$row['type'],
    'info' => &$row['info'],
    'optional' => &$row['optional'],
    'operations' => &$row['operations'],
  );

  $form['parameters']['table']['#rows']['add_new'] = array(
    'label' => array('data' => &$row['label']),
    'machine_name' => array('data' => &$row['machine_name']),
    'type' => array('data' => &$row['type']),
    'info' => array('data' => &$row['info']),
    'optional' => array('data' => &$row['optional']),
    'operations' => array('data' => &$row['operations']),
  );

  if (!empty($form_state['template']->entity_type) && !empty($form_state['template']->bundle)) {
    $form += $template->getBuilder()->configurationForm($form, $form_state);
  }
  else {
    $form['template'] = array(
      '#markup' => '<div id="template-form"></div>',
    );
  }

  $form['ui_settings'] = array(
    '#title' => t('Interface Settings'),
    '#description' => t('In this section you can configure the ui to create an entity from this template.'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#parents' => array('settings'),
    '#tree' => TRUE,
  );
  $form['ui_settings']['form'] = array(
    '#title' => t('Entity Form'),
    '#description' => t('The last step of creating an entity from a template is to edit and save the created entity. Use this select list to configure what form is used to edit the entity.'),
    '#type' => 'select',
    '#default_value' => !empty($template->settings['form']) ? $template->settings['form'] : '',
    '#options' => array(
      'entity_template:none' => t('No Form'),
      'entity_template:standard' => t('Standard'),
    ),
  );

  // If the flexiform module exists add flexiforms to the list.
  if (module_exists('flexiform') && !empty($template->entity_type) && !empty($template->bundle)) {
    $flexiforms = flexiform_get_base_entity_forms($template->entity_type, $template->bundle);
    foreach ($flexiforms as $flexiform) {
      $form['ui_settings']['form']['#options']['flexiform:'.$flexiform->form] = $flexiform->label;
    }
  }

  // Redirect settings.
  $form['ui_settings']['redirect'] = array(
    '#title' => t('Redirect'),
    '#description' => t('Where should the user be redirected to after creating an entity using this template. You may use tokens of the form %site:..., %current-user:... and %created-entity:.... Use &lt;none&gt; to force no redirect (preventing other modules from overriding the setting).'),
    '#type' => 'textfield',
    '#default_value' => !empty($template->settings['redirect']) ? $template->settings['redirect'] : '',
  );

  $form['revisions'] = array(
    '#title' => t('Revisions'),
    '#type' => 'fieldset',
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  $form['revisions']['log'] = array(
    '#title' => t('Log Message'),
    '#description' => t('A short message to describing the revision.'),
    '#type' => 'textarea',
    '#required' => TRUE,
    '#rows' => 3,
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 40,
    '#limit_validation_errors' => $limit_validation_errors,
  );

  return $form;
}

/**
 * Ajax Callback to Update type element.
 */
function entity_template_form_ajax_new_param_type($form, $form_state) {
  return $form['parameters']['table']['add_new']['info'];
}

/**
 * Form Validate Callback for making tasks.
 */
function entity_template_form_validate($form, &$form_state) {
  $template = clone $form_state['template'];
  field_attach_form_validate('entity_template', $template, $form['basic'], $form_state);

  if (!empty($form_state['template']->entity_type) && !empty($form_state['template']->bundle)) {
    $template->getBuilder()->configurationFormValidate($form, $form_state);
  }
}

/**
 * Form Submit Callback for making tasks.
 */
function entity_template_form_submit($form, &$form_state) {
  $template = $form_state['template'];
  $template->label = $form_state['values']['label'];
  $template->name = $form_state['values']['name'];
  $template->settings = $form_state['values']['settings'];
  $template->tags = drupal_explode_tags($form_state['values']['tags']);

  if (!empty($form_state['template']->entity_type) && !empty($form_state['template']->bundle)) {
    $builder = $template->getBuilder();
    $builder->configurationFormSubmit($form, $form_state);
  }

  if (!is_string($template->template)) {
    $template->template = drupal_json_encode($template->template);
  }

  // Attach Fields.
  field_attach_submit('entity_template', $template, $form['basic'], $form_state);

  // Set up revision stuff.
  $template->log = $form_state['values']['log'];
  $template->timestamp = REQUEST_TIME;
  $template->is_new_revision = TRUE;

  $template->save();
}

/**
 * Form validate callback for adding a parameter.
 */
function entity_template_form_validate_add_parameter($form, &$form_state) {
  $add_vals = $form_state['values']['parameters']['add_new'];
  $template = $form_state['template'];

  if (!empty($template->parameters[$add_vals['machine_name']])) {
    form_error($form['parameters']['table']['add_new']['machine_name'], t('A parameter with that name already exists.'));
  }

  $form_state['open_params_fieldset'] = TRUE;
}

/**
 * Form submit callback for adding a parameter.
 */
function entity_template_form_submit_add_parameter($form, &$form_state) {
  $add_vals = $form_state['values']['parameters']['add_new'];
  unset($add_vals['operations']);
  $template = $form_state['template'];
  $template->parameters[$add_vals['machine_name']] = $add_vals;

  $form_state['rebuild'] = TRUE;
  $form_state['open_params_fieldset'] = TRUE;
}

/**
 * Form submit callback for removing a parameter.
 */
function entity_template_form_submit_remove_parameter($form, &$form_state) {
  $button = $form_state['triggering_element'];
  $template = $form_state['template'];
  unset($template->parameters[$button['#param_key']]);

  $form_state['rebuild'] = TRUE;
  $form_state['open_params_fieldset'] = TRUE;
}

/**
 * Form Ajax Callback to reload the template fieldset.
 */
function entity_template_form_update_bundle($form, $form_state) {
  return $form['template'];
}

/**
 * Form Ajax Callback to reload the bundle select.
 */
function entity_template_form_update_entity_type($form, $form_state) {
  return $form['basic']['bundle'];
}

/**
 * Form Wrapper for the entity form part of the template form.
 */
function entity_template_entity_form_wrapper($form, &$form_state) {
  $form_id = $form_state['build_info']['form_id'];
  $args = $form_state['build_info']['args'];
  if (!function_exists($form_id)) {
    // In cases where many form_ids need to share a central constructor function,
    // such as the node editing form, modules can implement hook_forms(). It
    // maps one or more form_ids to the correct constructor functions.
    //
    // We cache the results of that hook to save time, but that only works
    // for modules that know all their form_ids in advance. (A module that
    // adds a small 'rate this comment' form to each comment in a list
    // would need a unique form_id for each one, for example.)
    //
    // So, we call the hook if $forms isn't yet populated, OR if it doesn't
    // yet have an entry for the requested form_id.
    if (!isset($forms) || !isset($forms[$form_id])) {
      $forms = module_invoke_all('forms', $form_id, $args);
    }
    $form_definition = $forms[$form_id];
    if (isset($form_definition['wrapper_callback']) && is_callable($form_definition['wrapper_callback'])) {
      $args = array_merge(array($form, &$form_state), $args);
      $form = call_user_func_array($form_definition['wrapper_callback'], $args);
    }
  }

  $form['#parents'] = array('template');
  return $form;
}

/**
 * Get the form to edit a particular entity type.
 *
 * This function makes a sandbox $form_state and build the form to edit the
 * entity.
 *
 * @see entity_ui_get_form
 */
function entity_template_get_entity_form($form, &$form_state, $entity_type, $entity, $op) {
  global $user;

  form_load_include($form_state, 'inc', 'rules', 'ui/ui.forms');

  if (isset($entity)) {
    [, , $bundle] = entity_extract_ids($entity_type, $entity);

    if (isset($entity->__template)) {
      unset($entity->__template);
    }
  }

  // Get property infor for the entity type and bundle.
  $entity_info = entity_get_info($entity_type);
  $bundle_key = !empty($entity_info['entity keys']['bundle']) ? $entity_info['entity keys']['bundle'] : NULL;
  $property_info = entity_get_property_info($entity_type);
  $properties = $property_info['properties'] ?? [];
  if (isset($bundle)) {
    $properties += $property_info['bundles'][$bundle]['properties'] ?? [];
  }

  $properties = array_filter($properties, fn($prop) => !empty($prop['setter callback']));

  $form_fields = [
    '#parents' => ['template'],
  ];
  field_attach_form($entity_type, clone $entity, $form_fields,$form_state, LANGUAGE_NONE);
  $wrapper = entity_metadata_wrapper($entity_type, $entity);

  $js_settings = array();
  foreach ($properties as $key => $property_info) {
    if ($key === $bundle_key) {
      $form[$key] = array(
        '#type' => 'value',
        '#value' => $bundle,
      );
      continue;
    }

    // Detect Access.
    $access = TRUE;
    if (!empty($info['access callback'])) {
      $access = call_user_func($info['access callback'], $op, $key, NULL, $user, $info['type']);
    }
    elseif (isset($info['setter permission'])) {
      $access = user_access($info['setter permission']);
    }

    // Work out #access.
    $js_settings[$key]['donotuse'] = !$access;
    $js_settings[$key]['required'] = !empty($property_info['required']);
    $js_settings[$key]['filled'] = !empty($entity->{$key}[LANGUAGE_NONE]) || !empty($entity->{$key}['select']) || !empty($entity->{$key}['value']);

    // Make sure nothing is required.
    if (!empty($form_fields[$key])) {
      foreach (element_children($form_fields[$key]) as $sub_key) {
        if (!empty($form_fields[$key][$sub_key]['#required'])) {
          $js_settings[$key]['required'] = TRUE;
          $form_fields[$key][$sub_key]['#required'] = FALSE;
        }

        foreach (element_children($form_fields[$key][$sub_key]) as $sub_sub_key) {
          if (!empty($form_fields[$key][$sub_key][$sub_sub_key]['#required'])) {
            $js_settings[$key]['required'] = TRUE;
            $form_fields[$key][$sub_key][$sub_sub_key]['#required'] = FALSE;
          }

          foreach (element_children($form_fields[$key][$sub_key][$sub_sub_key]) as $ssub_sub_key) {
            if (!empty($form_fields[$key][$sub_key][$sub_sub_key][$ssub_sub_key]['#required'])) {
              $js_settings[$key]['required'] = TRUE;
              $form_fields[$key][$sub_key][$sub_sub_key][$ssub_sub_key]['#required'] = FALSE;
            }
          }
        }
      }
    }

    $title = $property_info['label'];
    $form[$key.'--tab'] = array(
      '#type' => 'fieldset',
      '#title' => $title,
      '#attributes' => array(
        'class' => array(
          'entity-template-property-tab',
          'entity-template-property-'.$key,
        ),
        'property' => $key,
      ),
      '#weight' => ($form_fields[$key]['#weight'] ?? 0),
    );
    if ($js_settings[$key]['donotuse']) {
      $form[$key.'--tab']['#weight'] += 1000;
    }
    if ($js_settings[$key]['required']) {
      $form[$key.'--tab']['#weight'] -= 1000;
    }

    $form[$key.'--tab']['group'] = array(
      '#type' => 'container',
      '#attached' => array('library' => array(array('field_group', 'horizontal-tabs'))),
      //'#access' => $element[$key]['#access'], @todo: See if we need to reintroduce this.
      'method' => array(
        '#type' => 'radios',
        '#options' => array(
          'select' => t('Data Select'),
          'direct' => t('Direct Input'),
        ),
        '#default_value' => $entity->{$key}['method'] ?? (!empty($entity->{$key}['select']) ? 'select' : 'direct'),
        '#attributes' => array(
          'class' => array('entity-template-method-radios'),
        ),
      ),
      'parameter' => array(
        '#type' => 'container',
        '#title' => t('Data Select'),
        'select' => array(
          '#type' => 'rules_data_selection',
          '#title' => t('Select'),
          '#default_value' => $entity->{$key}['select'] ?? NULL,
          '#autocomplete_path' => "entity_template/selector/template/{$entity_type}/{$bundle}/{$key}",
          '#size' => 75,
        ),
        'select_template_options' => array(
          '#type' => 'fieldset',
          '#collapsible' => TRUE,
          '#title' => t('Options'),
        ),
        '#states' => array(
          'visible' => array(
            ':input[name="template['.$key.'--tab][group][method]"]' => array('value' => 'select'),
          ),
        ),
      ),
      'direct_input' => array(
        '#type' => 'container',
        '#title' => t('Direct Input'),
        'element' => NULL,
        'help' => NULL,
        '#states' => array(
          'visible' => array(
            ':input[name="template['.$key.'--tab][group][method]"]' => array('value' => 'direct'),
          ),
        ),
      ),
    );

    // Add parameter select options. These should do things like allow you to
    // offset dates and tweak numbers.
    // @see RulesDataProcessor::attachForm().
    $options_element = &$form[$key.'--tab']['group']['parameter']['select_template_options'];
    RulesDataProcessor::attachForm(
      $options_element,
      ($entity->{$key}['select_template_options'] ?? []) ?: [],
      $property_info,
      $form_state['template']->getParameters()
    );
    $options_element['#access'] = count(element_children($options_element));

    // Handle Direct Inputs.
    if (!empty($property_info['field'])) {
      $form[$key.'--tab']['group']['direct_input']['element'] = $form_fields[$key];
      $form[$key.'--tab']['group']['direct_input']['element']['#parents'] = ['template', $key];
    }
    else if (!empty($property_info['options list'])) {
      $form[$key.'--tab']['group']['direct_input']['element'] = [
        '#type' => 'select',
        '#title' => $property_info['label'],
        '#options' => call_user_func($property_info['options list'], $key, $property_info, 'edit'),
        '#empty_option' => t('- Select -'),
        '#default_value' => $entity->{$key}['value'] ?? NULL,
        '#multiple' => str_starts_with($property_info['type'], 'list<'),
      ];
    }
    else if (in_array($property_info['type'], array('text', 'decimal'))) {
      $form[$key.'--tab']['group']['direct_input']['element'] = [
        '#type' => 'textfield',
        '#title' => $property_info['label'],
        '#description' => $property_info['description'],
        '#default_value' => $entity->{$key}['value'] ?? NULL,
      ];
    }
    else if ($property_info['type'] === 'date') {
      $form[$key.'--tab']['group']['direct_input']['element'] = [
        '#type' => 'textfield',
        '#title' => $property_info['label'],
        '#description' => $property_info['description'],
        '#default_value' => $entity->{$key}['value'] ?? NULL,
      ];
    }
    else if ($prop_entity_info = entity_get_info($property_info['type'])) {
      $form[$key.'--tab']['group']['direct_input']['element'] = [
        '#title' => $property_info['label'],
        '#type' => 'textfield',
        '#description' => t('Enter the !entity_type ID.', array('!entity_type' => $prop_entity_info['label'])),
        '#default_value' => is_array($entity->{$key}) ? $entity->{$key}['value'] : $entity->{$key},
      ];
    }
    else {
      // Remove the direct input option.
      unset($form[$key.'--tab']['group']['direct_input']);
      unset($form[$key.'--tab']['group']['method']['#options']['direct']);
      $form[$key.'--tab']['group']['method']['#default_value'] =
        $form[$key.'--tab']['group']['method']['#default_value'] == 'direct' ?
          'select' :
          $form[$key.'--tab']['group']['method']['#default_value'];
    }

    if (in_array($property_info['type'], array('text', 'text_formatted', 'list<text>', 'list<text_formatted>'))) {
      $form[$key.'--tab']['group']['direct_input']['help'] = array(
        '#markup' => t('You may use tokens in the same format as rules.'),
      );
    }

    $sub_selectors[$key] = $entity->{$key}['sub_select'] ?? [];

    // Allow Data Select By Components.
    $sub_properties = array();
    if (!empty($property_info['property info'])) {
      $fc_wrapper = NULL;
      $sub_properties = $property_info['property info'];
    }
    else if (
      $property_info['type'] == 'field_collection_item' ||
      ($is_list = entity_property_list_extract_type($property_info['type'])) == 'field_collection_item'
    ) {
      $fc_wrapper = entity_metadata_wrapper('field_collection_item', NULL, $property_info);
      $sub_properties = $fc_wrapper->getPropertyInfo();
    }

    if (!empty($sub_properties)) {
      $form[$key.'--tab']['group']['method']['#options']['sub_select'] = t('Components Data Select');
      $parameter_sub = array(
        '#type' => 'container',
        '#title' => t('Components Data Select'),
        '#states' => array(
          'visible' => array(
            ':input[name="template['.$key.'--tab][group][method]"]' => array('value' => 'sub_select'),
          ),
        ),
      );
      if ($property_info['type'] == 'list<field_collection_item>') {
        $form_state['nested_items'][$key] = !empty($form_state['nested_items'][$key]) ? $form_state['nested_items'][$key] : 2;
        for ($i = 0; $i < $form_state['nested_items'][$key]; $i++) {
          $parameter_sub['items'][$i] = array(
            '#type' => 'fieldset',
            '#collapsible' => TRUE,
            '#collapsed' => TRUE,
            '#title' => t('Item !num', array('!num' => $i + 1)),
          );
          $parameter_sub['items'][$i]['remove_item'] = array(
            '#type' => 'submit',
            '#value' => t('Remove'),
            '#submit' => array('entity_template_form_component_data_select_remove_item'),
            '#nested_key' => $key,
            '#item_delta' => $i,
            '#ajax' => array(
              'wrapper' => $key.'-add-more-wrapper-parameter',
              'method' => 'replace',
              'callback' => 'entity_template_form_component_data_select_remove_item_ajax',
            ),
            '#limit_validation_errors' => array(),
          );

          foreach ($sub_properties as $sub_name => $sub_prop) {
            if (empty($sub_prop['setter callback'])) {
              continue;
            }

            // Exclude host entity property.
            if ($sub_name == 'host_entity') {
              continue;
            }

            $sub_prop['name'] = $sub_name;

            $parameter_sub['items'][$i][$sub_name] = array(
              '#type' => 'horizontal_tabs',
              '#attached' => array('library' => array(array('field_group', 'horizontal-tabs'))),
              'parameter' => array(
                '#type' => 'fieldset',
                '#title' => t('Data Select'),
                'select' => array(
                  '#type' => 'rules_data_selection',
                  '#title' => $sub_prop['label'],
                  '#default_value' => !empty($sub_selectors[$key][$i][$sub_name]['parameter']['select']) ? $sub_selectors[$key][$i][$sub_name]['parameter']['select'] : NULL,
                  '#autocomplete_path' => "entity_template/selector/template/{$entity_type}/{$bundle}/{$key}:{$sub_name}",
                  '#size' => 75,
                ),
                'select_template_options' => array(
                  '#type' => 'fieldset',
                  '#collapsible' => TRUE,
                  '#title' => t('Options'),
                ),
              ),
              'direct_input' => array(
                '#type' => 'fieldset',
                '#title' => t('Direct Input'),
                'element' => NULL,
              ),
            );

            // Add parameter select options. These should do things like allow you to
            // offset dates and tweak numbers.
            // @see RulesDataProcessor::attachForm().
            if (empty($sub_selectors[$key][$i][$sub_name]['parameter']['select_template_options'])) {
              $sub_selectors[$key][$i][$sub_name]['parameter']['select_template_options'] = array();
            }
            $options_element = &$parameter_sub['items'][$i][$sub_name]['parameter']['select_template_options'];
            RulesDataProcessor::attachForm(
              $options_element,
              ($sub_selectors[$key][$i][$sub_name]['parameter']['select_template_options'] ?? []) ?: [],
              $sub_prop,
              $form_state['template']->getParameters()
            );
            $options_element['#access'] = count(element_children($options_element));

            // Work out direct input
            if (!empty($sub_prop['options list'])) {
              $prop_wrapper = !empty($fc_wrapper) ? $fc_wrapper->{$sub_name} : entity_metadata_wrapper($sub_prop['type'], NULL, $sub_prop);
              $parameter_sub['items'][$i][$sub_name]['direct_input']['element'] = array(
                '#title' => $sub_prop['label'],
                '#type' => 'select',
                '#options' => $prop_wrapper->optionsList(),
                '#empty_option' => t('- Select -'),
                '#default_value' => $sub_selectors[$key][$i][$sub_name]['direct_input']['element'],
              );
            }
            else if ($sub_prop_entity_info = entity_get_info($sub_prop['type'])) {
              $parameter_sub['items'][$i][$sub_name]['direct_input']['element'] = array(
                '#title' => $sub_prop['label'],
                '#type' => 'textfield',
                '#description' => t('Enter the !entity_type ID.', array('!entity_type' => $sub_prop_entity_info['label'])),
                '#default_value' => $sub_selectors[$key][$i][$sub_name]['direct_input']['element'],
              );
            }
            else {
              $parameter_sub['items'][$i][$sub_name]['direct_input']['element'] = array(
                '#title' => $sub_prop['label'],
                '#type' => 'textfield',
                '#description' => t('Enter a !type value.', array('!type' => $sub_prop['type'])),
                '#default_value' => $sub_selectors[$key][$i][$sub_name]['direct_input']['element'],
              );
            }
          }
        }

        $parameter_sub['items']['#type'] = 'container';
        $parameter_sub['items']['#prefix'] = '<div id="'.$key.'-add-more-wrapper-parameter">';
        $parameter_sub['items']['#suffix'] = '</div>';
        $parameter_sub['items']['add_more'] = array(
          '#type' => 'submit',
          '#value' => t('Add Another Item'),
          '#submit' => array('entity_template_form_component_data_select_add_another'),
          '#nested_key' => $key,
          '#ajax' => array(
            'wrapper' => $key.'-add-more-wrapper-parameter',
            'method' => 'replace',
            'callback' => 'entity_template_form_component_data_select_add_another_ajax',
          ),
          '#limit_validation_errors' => array(),
        );
      }
      else {
        foreach ($sub_properties as $sub_name => $sub_prop) {
          if (empty($sub_prop['setter callback'])) {
            continue;
          }

          // Exclude host entity property.
          if ($sub_name == 'host_entity') {
            continue;
          }

          $sub_prop['name'] = $sub_name;

          $parameter_sub[$sub_name] = array(
            '#type' => 'horizontal_tabs',
            '#attached' => array('library' => array(array('field_group', 'horizontal-tabs'))),
            'parameter' => array(
              '#type' => 'fieldset',
              '#title' => t('Data Select'),
              'select' => array(
                '#type' => 'rules_data_selection',
                '#title' => $sub_prop['label'],
                '#default_value' => !empty($sub_selectors[$key][$sub_name]) ? (is_string($sub_selectors[$key][$sub_name]) ?  $sub_selectors[$key][$sub_name] : $sub_selectors[$key][$sub_name]['parameter']['select']) : NULL,
                '#autocomplete_path' => "entity_template/selector/template/{$entity_type}/{$bundle}/{$key}:{$sub_name}",
                '#size' => 75,
              ),
              'select_template_options' => array(
                '#type' => 'fieldset',
                '#collapsible' => TRUE,
                '#collapsed' => TRUE,
                '#title' => t('Options'),
              ),
            ),
            'direct_input' => array(
              '#type' => 'fieldset',
              '#title' => t('Direct Input'),
              'element' => NULL,
            ),
          );

          // Add parameter select options. These should do things like allow you to
          // offset dates and tweak numbers.
          // @see RulesDataProcessor::attachForm().
          if (empty($sub_selectors[$key][$sub_name]['parameter']['select_template_options'])) {
            $sub_selectors[$key][$sub_name]['parameter']['select_template_options'] = array();
          }

          $options_element = &$parameter_sub[$sub_name]['parameter']['select_template_options'];
          RulesDataProcessor::attachForm(
            $options_element,
            (!empty($sub_selectors[$key][$sub_name.'_template_options']) ?
              $sub_selectors[$key][$sub_name.'_template_options'] :
              (($sub_selectors[$key][$sub_name]['parameter']['select_template_options'] ?? []) ?: [])
            ),
            $sub_prop,
            $form_state['template']->getParameters()
          );
          $options_element['#access'] = count(element_children($options_element));

          // Work out direct input
          if (!empty($sub_prop['options list'])) {
            $prop_wrapper = !empty($fc_wrapper) ? $fc_wrapper->{$sub_name} : entity_metadata_wrapper($sub_prop['type'], NULL, $sub_prop);
            $parameter_sub[$sub_name]['direct_input']['element'] = array(
              '#title' => $sub_prop['label'],
              '#type' => 'select',
              '#empty_option' => t('- Select -'),
              '#options' => $prop_wrapper->optionsList(),
              '#default_value' => !empty($sub_selectors[$key][$sub_name]['direct_input']['element']) ? $sub_selectors[$key][$sub_name]['direct_input']['element'] : NULL,
            );
          }
          else if ($sub_prop_entity_info = entity_get_info($sub_prop['type'])) {
            $parameter_sub[$sub_name]['direct_input']['element'] = array(
              '#title' => $sub_prop['label'],
              '#type' => 'textfield',
              '#description' => t('Enter the !entity_type ID.', array('!entity_type' => $sub_prop_entity_info['label'])),
              '#default_value' => !empty($sub_selectors[$key][$sub_name]['direct_input']['element']) ? $sub_selectors[$key][$sub_name]['direct_input']['element'] : NULL,
            );
          }
          else {
            $parameter_sub[$sub_name]['direct_input']['element'] = array(
              '#title' => $sub_prop['label'],
              '#type' => 'textfield',
              '#description' => t('Enter a !type value.', array('!type' => $sub_prop['type'])),
              '#default_value' => !empty($sub_selectors[$key][$sub_name]['direct_input']['element']) ? $sub_selectors[$key][$sub_name]['direct_input']['element'] : NULL,
            );
          }
        }
      }

      if (count(element_children($parameter_sub))) {
        $form[$key.'--tab']['group']['parameter_sub'] = $parameter_sub;
      }
    }
  }

  $form['#attached']['css'][] = drupal_get_path('module', 'entity_template') . '/css/entity_template.admin.css';
  $form['#attached']['js'][] = drupal_get_path('module', 'entity_template') . '/js/entity_template.admin.js';
  $form['#attached']['js'][] = array(
    'type' => 'setting',
    'data' => array(
      'entity_template' => $js_settings,
    ),
  );

  drupal_alter('entity_template_get_entity_form', $form, $form_state, $entity_type, $entity);

  foreach ($form['#attached']['js'] as $k => $info) {
    if (is_array($info) && $info['type'] == 'setting' && key($info['data']) == 'entity_template') {
      $js_settings = $form['#attached']['js'][$k]['data']['entity_template'];
    }
  }
  foreach (element_children($form) as $element_key) {
    [$key,] = explode('--', $element_key);
    if ($js_settings[$key]['donotuse']) {
      $form[$element_key]['#weight'] += 1000;
    }
    if ($js_settings[$key]['required']) {
      $form[$element_key]['#weight'] -= 1000;
    }
  }

  $form['#after_build'][] = 'entity_template_entity_form_after_build';

  return $form;
}

/**
 * After build that removes the conditional fields.
 */
function entity_template_entity_form_after_build($element, &$form_state) {
  $form = &$form_state['complete form'];
  unset($form['#conditional_fields']);
  return $element;
}

/**
 * Submit callback to remove and item from components data select.
 */
function entity_template_form_component_data_select_remove_item($form, &$form_state) {
  $element = $form_state['triggering_element'];
  $key = $element['#nested_key'];
  $delta = $element['#item_delta'];
  $parents = $element['#parents'];
  array_pop($parents); array_pop($parents);

  if ($input = &drupal_array_get_nested_value($form_state['input'], $parents)) {
    unset($input[$delta]);
    $input = array_values($input);
  }
  if ($values = &drupal_array_get_nested_value($form_state['values'], $parents)) {
    unset($values[$delta]);
    $values = array_values($values);
  }

  $form_state['nested_items'][$key]--;
  $form_state['rebuild'] = TRUE;
}

/**
 * Ajax Callback to refresh components data select.
 */
function entity_template_form_component_data_select_remove_item_ajax($form, $form_state) {
  $element = $form_state['triggering_element'];
  $array_parents = $element['#array_parents'];
  array_pop($array_parents); array_pop($array_parents);

  return drupal_array_get_nested_value($form, $array_parents);
}

/**
 * Submit callback to add another item to components data select.
 */
function entity_template_form_component_data_select_add_another($form, &$form_state) {
  $element = $form_state['triggering_element'];
  $key = $element['#nested_key'];
  $form_state['nested_items'][$key]++;
  $form_state['rebuild'] = TRUE;
}

/**
 * Ajax Callback to refresh components data select.
 */
function entity_template_form_component_data_select_add_another_ajax($form, $form_state) {
  $element = $form_state['triggering_element'];
  $array_parents = $element['#array_parents'];
  array_pop($array_parents);

  return drupal_array_get_nested_value($form, $array_parents);
}

/**
 * Return options for the data selector.
 */
function entity_template_selector_autocomplete($template_address, $entity_type, $bundle, $key, $build_id, $string = '') {
  // Split key if required.
  if (strpos($key, ':')) {
    [$key, $sub_key] = explode(':', $key, 2);
  }

  // Get the form and its state.
  $form_state = form_state_defaults();
  $form = form_get_cache($build_id, $form_state);

  // Get property info for the entity type and bundle.
  if (entity_get_info($entity_type)) {
    $property_info = entity_get_property_info($entity_type);
    $properties = $property_info['properties'];
    if (isset($bundle)) {
      $properties += $property_info['bundles'][$bundle]['properties'];
    }
  }
  else if ($entity_type === 'struct') {
    $properties = $form_state['entity_template_selector_structs'][$key];
  }

  // Get available variables
  $template_address = $template_address === 'null' ? ['template'] : explode(':', $template_address);
  /** @var \EntityTemplate|\TemplateConfiguration $template */
  $template = drupal_array_get_nested_value($form_state, $template_address);
  $variables = $template instanceof EntityTemplate ? $template->getParameters() : $template->getParametersInfo();

  // Convert variables into a format rules understands.
  foreach ($variables as $name => $var) {
    if (!empty($var['type'])) {
      continue;
    }

    $variables[$name]['type'] = $var['entity_type'];
  }

  if (isset($properties[$key])) {
    $selectors = explode(',', $string);
    $current_string = array_pop($selectors);
    $prefix = '';
    if (count($selectors)) {
      $prefix = implode(',', $selectors).',';
    }

    $parts = explode(':', $current_string);
    $last_part = array_pop($parts);
    $selector = implode(':', $parts);

    // Get the property we're matching for.
    $property = $properties[$key];
    if (!empty($sub_key) && !empty($property['property info'][$sub_key])) {
      $property = $property['property info'][$sub_key];
    }

    // Start with the partly given selector from scratch.
    $result = array();
    if ($selector && $wrapper = entity_template_apply_selector($variables, $selector)) {
      $result = RulesData::matchingDataSelector($wrapper, $property, $selector . ':', 0);
      if ($list_type = entity_property_list_extract_type($property['type'])) {
        $tmp_property = $property;
        $tmp_property['type'] = $list_type;
        $result += RulesData::matchingDataSelector($wrapper, $tmp_property, $selector . ':', 0);
      }
    }
    elseif (!$selector) {
      $result = RulesData::matchingDataSelector($variables, $property, '', 0);
      if ($list_type = entity_property_list_extract_type($property['type'])) {
        $tmp_property = $property;
        $tmp_property['type'] = $list_type;
        $result += RulesData::matchingDataSelector($variables, $tmp_property, '', 0);
      }
    }

    foreach ($result as $selector => $info) {
      // If we have an uncomplete last part, take it into account now.
      $attributes = array();
      if (!$last_part || strpos($selector, $string) === 0) {
        $attributes['class'][] = 'rules-dsac-item';
        $attributes['title'] = isset($info['description']) ? strip_tags($info['description']) : '';
        if ($selector[strlen($selector) - 1] == ':') {
          $attributes['class'][] = 'rules-dsac-group';
          $text = check_plain($selector) . '... (' . check_plain($info['label']) . ')';
        }
        else {
          $text = check_plain($selector) . ' (' . check_plain($info['label']) . ')';
        }
        $matches[$prefix.$selector] = "<div" . drupal_attributes($attributes) . ">$text</div>";
      }
    }
  }
  drupal_json_output($matches);
}

/**
 * Apply data selectors.
 *
 * @see RulesPlugin::applyDataSelector
 */
function entity_template_apply_selector($vars, $selector) {
  $parts = explode(':', str_replace('-', '_', $selector), 2);
  if (isset($vars[$parts[0]]['type'])) {
    $wrapper = rules_wrap_data(NULL, $vars[$parts[0]], TRUE);
    if (count($parts) > 1 && $wrapper instanceof EntityMetadataWrapper) {
      try {
        foreach (explode(':', $parts[1]) as $name) {
          if ($wrapper instanceof EntityListWrapper || $wrapper instanceof EntityStructureWrapper) {
            $wrapper = $wrapper->get($name);
          }
          else {
            return FALSE;
          }
        }
      }
      // In case of an exception or we were unable to get a wrapper, return FALSE.
      catch (EntityMetadataWrapperException $e) {
        return FALSE;
      }
    }
  }
  return isset($wrapper) ? $wrapper : FALSE;
}
